# 单点保存工具 README

## 简介
L0, L1, mix dump存在盲区，网络中的非api/module的输入输出不会被批量dump下来。单点保存提供类似np.save和print的功能和使用体验，可以保存指定的变量。同时针对大模型场景进行了增强，具备以下特性：
- 可保存变量的反向梯度结果。
- 能直接保存嵌套结构数据（如 list、dict），无需手动遍历。
- 自动分 rank 保存。
- 多次调用时会自动计数。
- 可配置保存统计值或者张量。

## 支持场景
仅支持 PyTorch 与 MindSpore 的动态图场景。

## 使能方式

### 配置文件说明

通用配置：

| 参数     | 解释                                       | 是否必选 |
| -------- |-------------------------------------------| -------- |
| task     | dump 的任务类型，str 类型。 单点保存场景仅支持传入"statistics", "tensor"。    |  是     |
| level    | dump 级别，str 类型，根据不同级别采集不同数据。单点保存场景传入"debug"。  | 是  |
| dump_path  | 设置 dump 数据目录路径，str 类型。细节详见[通用配置说明](./02.config_introduction.md#11-通用配置)  | 是       |
| rank        | 指定对某张卡上的数据进行采集，list[Union[int, str]] 类型。细节详见[通用配置说明](./02.config_introduction.md#11-通用配置)  | 否       |

"statistics" 任务子配置项：
| 参数     | 解释                                       | 是否必选 |
| -------- |-------------------------------------------| -------- |
| summary_mode     | 控制 dump 文件输出的模式，str 类型。支持传入"statistics", "md5"。 细节详见[statistics任务子配置项说明](./02.config_introduction.md#12-task-配置为-statistics)    |  否     |

"tensor" 任务无子配置项。

### 接口调用说明

调用PrecisionDebugger.save，传入需要保存的变量，指定变量名称以及是否需要保存反向数据。接口入参说明详见[pytorch单点保存接口](./05.data_dump_PyTorch.md#19-save)，[mindspore单点保存接口](./06.data_dump_MindSpore.md#615-save)

### 实例（以pytorch场景为例）

配置文件
```json
{
    "task": "statistics",
    "dump_path": "./dump_path",
    "rank": [],
    "level": "debug",
    "statistics": {
        "summary_mode": "statistics"
    }
}
```

初始化
```python
# 训练启动py脚本
from mindspore.pytorch import PrecisionDebugger
debugger = PrecisionDebugger("./config.json")
for data, label in data_loader:
    # 执行模型训练
    train(data, label)

```

初始化（无配置文件）
```python
# 训练启动py脚本
from mindspore.pytorch import PrecisionDebugger
debugger = PrecisionDebugger(dump_path="dump_path", level="debug")
for data, label in data_loader:
    # 执行模型训练
    train(data, label)

```

调用保存接口
```python
# 训练过程中被调用py文件
from mindspore.pytorch import PrecisionDebugger
dict_variable = {"key1": "value1", "key2": [1, 2]}
PrecisionDebugger.save(dict_variable, "dict_variable", save_backward=False)

```

## 输出结果
  * **"task" 配置为 "statistics" 场景** ：在 dump 目录下会生成包含变量统计值信息的 `debug.json` 文件。
  * **"task" 配置为 "tensor" 场景** ：除了在 dump 目录下生成包含变量统计值信息的 `debug.json` 文件外，还会在 dump 子目录 `dump_tensor_data` 中保存张量二进制文件，文件名称格式为 `{variable_name}{grad_flag}.{count}.tensor.{indexes}.{file_suffix}`。

    - variable_name： 传入save接口的变量名称。
    - grad_flag： 反向数据标识，反向数据为"_grad"，正向数据为""。
    - count： 调用计数，多次以相同变量名称调用时的计数。
    - indexes： 索引，在保存嵌套结构数据时的索引。例如：嵌套结构为`{"key1": "value1", "key2": ["value2", "value3"]}`，"value2"的索引为"key2.0"
    - file_suffix：文件后缀，pytorch场景为"pt"，mindspore场景为"npy"


